@@ -10,7 +10,8 @@
 
 #include <qtimer.h>
 #include <qptrlist.h>
-#include <qdialog.h>
+
+#include <kdialogbase.h>
 #include <kscreensaver.h>
 #include <krandomsequence.h>
 
@@ -48,11 +49,11 @@ protected:
 	KRandomSequence rnd;
 };
 
-class kPolygonSetup : public QDialog
+class kPolygonSetup : public KDialogBase
 {
 	Q_OBJECT
 public:
-	kPolygonSetup( QWidget *parent = NULL, const char *name = NULL );
+	kPolygonSetup( QWidget *parent = 0, const char *name = 0 );
     ~kPolygonSetup();
 
 protected:
@@ -62,8 +63,8 @@ private slots:
 	void slotLength( int );
 	void slotVertices( int );
 	void slotSpeed( int );
-	void slotOkPressed();
-	void slotAbout();
+	void slotOk();
+	void slotHelp();
 
 private:
 	QWidget *preview;
