@@ -8,7 +8,7 @@
 //
 // Developed by Georg Drenkhahn, georg-d@users.sourceforge.net
 //
-// $Id: rotation.cpp 376058 2005-01-07 11:14:10Z mlaurent $
+// $Id: rotation.cpp 435075 2005-07-16 08:53:15Z mueller $
 //
 /*
  * Copyright (C) 2004 Georg Drenkhahn
@@ -23,7 +23,7 @@
  *
  * You should have received a copy of the GNU General Public License along with
  * this program; if not, write to the Free Software Foundation, Inc., 59 Temple
- * Place, Suite 330, Boston, MA 02111-1307 USA
+ * Place, Suite 330, Boston, MA 02110-1301 USA
  */
 //============================================================================
 
@@ -81,15 +81,15 @@ extern "C"
 //-----------------------------------------------------------------------------
 
 EulerOdeSolver::EulerOdeSolver(
-   const double &_t,
-   const double &_dt,
-   const double &_A,
-   const double &_B,
-   const double &_C,
-   std::valarray<double> &_y,
-   const double &_eps)
-   : RkOdeSolver<double>(_t,_y,_dt,_eps),
-     A(_A), B(_B), C(_C)
+   const double &t_,
+   const double &dt_,
+   const double &A_,
+   const double &B_,
+   const double &C_,
+   std::valarray<double> &y_,
+   const double &eps_)
+   : RkOdeSolver<double>(t_,y_,dt_,eps_),
+     A(A_), B(B_), C(C_)
 {
 }
 
@@ -141,9 +141,9 @@ std::valarray<double> EulerOdeSolver::f(
 RotationGLWidget::RotationGLWidget(
    QWidget* parent, const char* name,
    const vec3<double>& _omega,
-   const std::deque<vec3<double> >& _e1,
-   const std::deque<vec3<double> >& _e2,
-   const std::deque<vec3<double> >& _e3,
+   const std::deque<vec3<double> >& e1_,
+   const std::deque<vec3<double> >& e2_,
+   const std::deque<vec3<double> >& e3_,
    const vec3<double>& J)
    : QGLWidget(parent, name),
      eyeR(25), eyeTheta(1), eyePhi(M_PI*0.25),
@@ -154,9 +154,9 @@ RotationGLWidget::RotationGLWidget(
      bodyAxsesLength(6),
      fixedAxsesLength(8),
      omega(_omega),
-     e1(_e1),
-     e2(_e2),
-     e3(_e3)
+     e1(e1_),
+     e2(e2_),
+     e3(e3_)
 {
    // set up initial rotation matrix as unit matrix, only non-constant elements
    // are set later on
