@@ -1,5 +1,5 @@
 /*
-  $Id: icewm.cpp 409718 2005-05-05 12:52:18Z lunakl $
+  $Id: icewm.cpp 465369 2005-09-29 14:33:08Z mueller $
 
   Gallium-IceWM themeable KWin client
 
@@ -19,8 +19,8 @@
 
   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
-  the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-  Boston, MA 02111-1307, USA.
+  the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+  Boston, MA 02110-1301, USA.
 
   -----------------------------------------------------------------------------
   This client loads most icewm 1.0.X pixmap themes, without taking into account
