@@ -83,7 +83,7 @@ QMap<unsigned int, QIntDict<GradientSet> > gradients; // gradients[color][size]
 
 PhaseStyle::PhaseStyle()
     : KStyle(FilledFrameWorkaround | AllowMenuTransparency,
-             ThreeButtonScrollBar), hover_(0),
+             ThreeButtonScrollBar), hover_(0), hovertab_(0),
       gradients_(QPixmap::defaultDepth() > 8), kicker_(false)
 {
     QSettings settings;
@@ -859,10 +859,22 @@ void PhaseStyle::drawPrimitive(PrimitiveElement element,
 
       case PE_PanelMenuBar:
       case PE_PanelDockWindow:
-          --x; --y; ++w; ++h; // adjust rect so we can use bevel
-          drawPhaseBevel(painter, x, y, w, h,
-                         group, group.background(), false, (w < h),
-                         (element==PE_PanelDockWindow) ? true : false);
+          if (kicker_ && (w == 2)) { // kicker handle separator
+              painter->setPen(group.mid());
+              painter->drawLine(x, y,  x, y2);
+              painter->setPen(group.midlight());
+              painter->drawLine(x+1, y,  x+1, y2);
+          } else if (kicker_ && (h == 2)) { // kicker handle separator
+              painter->setPen(group.mid());
+              painter->drawLine(x, y,  x2, y);
+              painter->setPen(group.midlight());
+              painter->drawLine(x, y+1,  x2, y+1);
+          } else {
+              --x; --y; ++w; ++h; // adjust rect so we can use bevel
+              drawPhaseBevel(painter, x, y, w, h,
+                             group, group.background(), false, (w < h),
+                             (element==PE_PanelDockWindow) ? true : false);
+          }
           break;
 
       case PE_DockWindowSeparator: {
@@ -1032,26 +1044,19 @@ void PhaseStyle::drawKStylePrimitive(KStylePrimitive element,
 
       //case KPE_DockWindowHandle:
       case KPE_GeneralHandle:
-          if (horiz) { // horizontal
-              painter->fillRect(rect, group.brush(QColorGroup::Background));
+          cx-=2; cy-=2;
+          painter->fillRect(rect, group.brush(QColorGroup::Background));
 
-              painter->setPen(group.mid());
-              painter->drawLine(cx-2, y, cx-2, y2-1);
-              painter->drawLine(cx+1, y, cx+1, y2-1);
-
-              painter->setPen(group.midlight());
-              painter->drawLine(cx-1, y+1, cx-1, y2);
-              painter->drawLine(cx+2, y+1, cx+2, y2);
-          } else { // vertical
-              painter->fillRect(rect, group.brush(QColorGroup::Background));
-
-              painter->setPen(group.mid());
-              painter->drawLine(x, cy-2, x2-1, cy-2);
-              painter->drawLine(x, cy+1, x2-1, cy+1);
-
-              painter->setPen(group.midlight());
-              painter->drawLine(x+1, cy-1, x2, cy-1);
-              painter->drawLine(x+1, cy+2, x2, cy+2);
+          if (horiz) {
+              for (n = -5; n <= 5; n += 5) {
+                  kColorBitmaps(painter, group, cx, cy+n,
+                                0, &doodad_mid, &doodad_light, 0, 0, 0);
+              }
+          } else {
+              for (n = -5; n <= 5; n += 5) {
+                  kColorBitmaps(painter, group, cx+n, cy,
+                                0, &doodad_mid, &doodad_light, 0, 0, 0);
+              }
           }
           break;
 
@@ -1478,8 +1483,7 @@ void PhaseStyle::drawControl(ControlElement element,
       case CE_TabBarTab: {
           const QTabBar* tab = dynamic_cast<const QTabBar*>(widget);
           if (tab) {
-              if ((widget == hover_) &&
-                  (tab->selectTab(mousepos_) == option.tab())) {
+              if ((widget == hover_) && (option.tab() == hovertab_)) {
                   flags |= Style_MouseOver;
               }
               // this guy can get complicated, we we do it elsewhere
@@ -1721,8 +1725,8 @@ void PhaseStyle::drawComplexControl(ComplexControl control,
               srect = querySubControlMetrics(control, widget,
                                              SC_ScrollBarAddPage, option);
               if (srect.isValid()) {
-                  if (horizontal) srect.addCoords(1, 0, 0, 0);
-                  else srect.addCoords(0, 1, 0, 0);
+                  if (horizontal) srect.addCoords(1, 0, 1, 0);
+                  else srect.addCoords(0, 1, 0, 1);
                   drawPrimitive(PE_ScrollBarAddPage, painter, srect, group,
                                 scrollflags | ((active == SC_ScrollBarAddPage)
                                                ? Style_Down : Style_Default));
@@ -1742,8 +1746,14 @@ void PhaseStyle::drawComplexControl(ComplexControl control,
 
           // slider
           if (controls & SC_ScrollBarSlider) {
-              srect = querySubControlMetrics(control, widget,
-                                             SC_ScrollBarSlider, option);
+              if (sb->minValue() == sb->maxValue()) {
+                  // maxed out
+                  srect = querySubControlMetrics(control, widget,
+                                                 SC_ScrollBarGroove, option);
+              } else {
+                  srect = querySubControlMetrics(control, widget,
+                                                 SC_ScrollBarSlider, option);
+              }
               if (srect.isValid()) {
                   if (horizontal) srect.addCoords(0, 0, 1, 0);
                   else srect.addCoords(0, 0, 0, 1);
@@ -1991,6 +2001,26 @@ int PhaseStyle::pixelMetric(PixelMetric metric, const QWidget *widget) const
     }
 }
 
+//////////////////////////////////////////////////////////////////////////////
+// subRect()
+// ---------
+// Return subrect for the widget in logical coordinates
+
+QRect PhaseStyle::subRect(SubRect rect, const QWidget *widget) const
+{
+    switch (rect) {
+      case SR_ComboBoxFocusRect: {
+          QRect r = querySubControlMetrics(CC_ComboBox, widget,
+                                           SC_ComboBoxEditField);
+          r.addCoords(1, 1,-1,-1);
+          return r;
+      }
+
+      default:
+          return KStyle::subRect(rect, widget);
+    }
+}
+
 //////////////////////////////////////////////////////////////////////////////
 // querySubControlMetrics()
 // ------------------------
@@ -2036,80 +2066,17 @@ QRect PhaseStyle::querySubControlMetrics(ComplexControl control,
       }
 
       case CC_ScrollBar: {
-          // NOTE: SC_ScrollBarFirst and SC_ScrollBarLast don't exist
           const QScrollBar *sb = dynamic_cast<const QScrollBar*>(widget);
           if (!sb) break;
 
-          bool horizontal;      // orientation of scrollbar
-          unsigned length;      // length of scrollbar minus buttons
-          unsigned extent;      // thickness of the scrollbar
-          unsigned sliderstart; // starting position of the slider
-          unsigned sliderlen, slidermin, range;
+          bool horizontal = (sb->orientation() == Qt::Horizontal);
+          rect = KStyle::querySubControlMetrics(control, widget,
+                                                subcontrol, option);
 
-          horizontal = (sb->orientation() == Qt::Horizontal);
-          extent = pixelMetric(PM_ScrollBarExtent, widget);
-          length = (horizontal ? w : h) - (extent*3) + 1;
-          sliderstart = sb->sliderStart();
-          slidermin = pixelMetric(PM_ScrollBarSliderMin, widget);
-
-          if (sb->maxValue() != sb->minValue()) {
-              range = sb->maxValue() - sb->minValue();
-              sliderlen = (sb->pageStep() * length)
-                  / (range + sb->pageStep());
-              if ((sliderlen < slidermin) || (range > INT_MAX / 2))
-                  sliderlen = slidermin;
-              if (sliderlen > length)
-                  sliderlen = length;
-          } else {
-              sliderlen = length;
-          }
-
-          // subcontrols
-          switch (subcontrol) {
-            case SC_ScrollBarSubLine: // top/left button
-                rect.setRect(0, 0, extent, extent);
-                break;
-
-            case SC_ScrollBarAddLine: // bottom/right button
-                if (horizontal)
-                    rect.setRect(w-extent, 0, extent, extent);
-                else
-                    rect.setRect(0, sb->height() - extent, extent, extent);
-                break;
-
-            case SC_ScrollBarSubPage: // between top/left button and slider
-                if (horizontal)
-                    rect.setRect(extent, 0, sliderstart-extent, extent);
-                else
-                    rect.setRect(0, extent, extent, sliderstart-extent);
-                break;
-
-            case SC_ScrollBarAddPage: // between bottom/right button and slider
-                if (horizontal)
-                    rect.setRect(sliderstart+sliderlen, 0,
-                                 length-sliderstart-sliderlen+extent, extent);
-                else
-                    rect.setRect(0, sliderstart+sliderlen, extent,
-                                 length-sliderstart-sliderlen+extent);
-                break;
-
-            case SC_ScrollBarGroove: // groove
-                if (horizontal)
-                    rect.setRect(extent-1, 0, length+1, h);
-                else
-                    rect.setRect(0, extent-1, w, length+1);
-                break;
-
-            case SC_ScrollBarSlider: // thumb/handle
-                if (horizontal)
-                    rect.setRect(sliderstart, 0, sliderlen, extent);
-                else
-                    rect.setRect(0, sliderstart, extent, sliderlen);
-                break;
-
-            default:
-                rect = QRect(); // invalidate
-                break;
+          // adjust the standard metrics so controls can "overlap"
+          if (subcontrol == SC_ScrollBarGroove) {
+              if (horizontal) rect.addCoords(-1, 0, 1, 0);
+              else            rect.addCoords(0, -1, 0, 1);
           }
           break;
       }
@@ -2252,6 +2219,7 @@ QSize PhaseStyle::sizeFromContents(ContentsType contents,
 bool PhaseStyle::flatToolbar(const QToolBar *toolbar) const
 {
     if (!toolbar) return true; // not on a toolbar
+    if (!toolbar->isMovingEnabled()) return true; // immobile toolbars are flat
     if (!toolbar->area()) return true; // not docked
     if (toolbar->place() == QDockWindow::OutsideDock) return true; // ditto
     if (!toolbar->mainWindow()) return true; // not in a main window
@@ -2282,16 +2250,17 @@ bool PhaseStyle::eventFilter(QObject *object, QEvent *event)
         // KDE Toolbar Widget
         // patch by Daniel Brownlees <dbrownlees@paradise.net.nz>
         if (object->parent() && !qstrcmp(object->name(), KTOOLBARWIDGET)) {
-            widget = static_cast<QWidget*>(object);
-            QWidget *parent = static_cast<QWidget*>(object->parent());
+            if (0 == (widget = dynamic_cast<QWidget*>(object))) return false;
+            QWidget *parent = dynamic_cast<QWidget*>(object->parent());
             int px = widget->x(), py = widget->y();
             // find the toolbar
             while (parent && parent->parent()
                    && !dynamic_cast<QToolBar*>(parent)) {
                 px += parent->x();
                 py += parent->y();
-                parent = static_cast<QWidget*>(parent->parent());
+                parent = dynamic_cast<QWidget*>(parent->parent());
             }
+            if (!parent) return false;
             widget->rect().rect(&x, &y, &w, &h);
             QRect prect = parent->rect();
 
@@ -2320,7 +2289,7 @@ bool PhaseStyle::eventFilter(QObject *object, QEvent *event)
         // QToolBarExtensionWidget
         else if ((object->parent()) &&
                  (toolbar = dynamic_cast<QToolBar*>(object->parent()))) {
-            widget = static_cast<QWidget*>(object);
+            if (0 == (widget = dynamic_cast<QWidget*>(object))) return false;
             horiz = (toolbar->orientation() == Qt::Horizontal);
             QPainter painter(widget);
             widget->rect().rect(&x, &y, &w, &h);
@@ -2350,7 +2319,7 @@ bool PhaseStyle::eventFilter(QObject *object, QEvent *event)
         }
 
         // QFrame lines (do this guy last)
-        else if ((frame = dynamic_cast<QFrame*>(object))) {
+        else if (0 != (frame = dynamic_cast<QFrame*>(object))) {
             QFrame::Shape shape = frame->frameShape();
             switch (shape) {
               case QFrame::HLine:
@@ -2378,26 +2347,44 @@ bool PhaseStyle::eventFilter(QObject *object, QEvent *event)
             ::qt_cast<QCheckBox*>(object) ||
             ::qt_cast<QRadioButton*>(object) ||
             ::qt_cast<QSlider*>(object) ||
-            ::qt_cast<QTabBar*>(object) ||
             object->inherits(QSPLITTERHANDLE)) {
             if (event->type() == QEvent::Enter) {
-                if ((widget = dynamic_cast<QWidget*>(object)) &&
+                if (0 != (widget = dynamic_cast<QWidget*>(object)) &&
                     widget->isEnabled()) {
                     hover_ = widget;
-                    mousepos_ = QPoint(0,0);
                     widget->repaint(false);
                 }
             } else if (event->type() == QEvent::Leave) {
                 if (0 != (widget = dynamic_cast<QWidget*>(object))) {
                     hover_ = 0;
-                    mousepos_ = QPoint(0,0);
+                    widget->repaint(false);
+                }
+            }
+        } else if (::qt_cast<QTabBar*>(object)) { // special case for qtabbar
+            if (event->type() == QEvent::Enter) {
+                if (0 != (widget = dynamic_cast<QWidget*>(object)) &&
+                    widget->isEnabled()) {
+                    hover_ = widget;
+                    hovertab_ = 0;;
+                    widget->repaint(false);
+                }
+            } else if (event->type() == QEvent::Leave) {
+                if (0 != (widget = dynamic_cast<QWidget*>(object))) {
+                    hover_ = 0;
+                    hovertab_ = 0;;
                     widget->repaint(false);
                 }
             } else if (event->type() == QEvent::MouseMove) {
-                if (0 != (widget = dynamic_cast<QWidget*>(object))) {
-                    QMouseEvent *me = dynamic_cast<QMouseEvent*>(event);
-                    mousepos_ = me->pos();
-                    widget->repaint(false);
+                QTabBar *tabbar;
+                if (0 != (tabbar = dynamic_cast<QTabBar*>(object))) {
+                    QMouseEvent *me;
+                    if (0 != (me = dynamic_cast<QMouseEvent*>(event))) {
+                        QTab *tab = tabbar->selectTab(me->pos());
+                        if (hovertab_ != tab) {
+                            hovertab_ = tab;
+                            tabbar->repaint(false);
+                        }
+                    }
                 }
             }
         }
